@@ -1,7 +1,7 @@
 import asyncio
 import functools
 import logging
-from typing import Any, Callable, Coroutine, Dict, Iterable, List, NoReturn, Optional
+from typing import Any, Callable, Coroutine, Dict, Iterable, List, Optional
 
 import aiopg
 import psycopg2
@@ -9,7 +9,7 @@ import psycopg2.errors
 import psycopg2.sql
 from psycopg2.extras import Json, RealDictCursor
 
-from procrastinate import connector, exceptions, sql, utils
+from procrastinate import connector, exceptions, sql
 
 logger = logging.getLogger(__name__)
 
@@ -20,8 +20,9 @@ CoroutineFunction = Callable[..., Coroutine]
 
 def wrap_exceptions(coro: CoroutineFunction) -> CoroutineFunction:
     """
-    Wrap psycopg2 and aiopg errors as connector exceptions
-    This decorator is expected to be used on coroutine functions only
+    Wrap psycopg2 and aiopg errors as connector exceptions.
+
+    This decorator is expected to be used on coroutine functions only.
     """
 
     @functools.wraps(coro)
@@ -39,8 +40,44 @@ def wrap_exceptions(coro: CoroutineFunction) -> CoroutineFunction:
     return wrapped
 
 
-@utils.add_sync_api
-class AiopgConnector(connector.BaseConnector):
+def wrap_query_exceptions(coro: CoroutineFunction) -> CoroutineFunction:
+    """
+    Detect aiopg OperationalError's with a "server closed the connection unexpectedly"
+    message and retry a number of times.
+
+    This is to handle the case where the database connection (obtained from the pool)
+    was actually closed by the server. In this case, aiopg raises an OperationalError
+    with a "server closed the connection unexpectedly" message (and no pgcode) when the
+    connection is used for issuing a query. What we do is retry when an OperationalError
+    is raised, and until the maximum number of retries is reached.
+
+    The number of retries is set to the pool maximum size plus one, to handle the case
+    where the connections we have in the pool were all closed on the server side.
+    """
+
+    @functools.wraps(coro)
+    async def wrapped(*args, **kwargs):
+        final_exc = None
+        try:
+            max_tries = args[0]._pool.maxsize + 1
+        except Exception:
+            max_tries = 1
+        for _ in range(max_tries):
+            try:
+                return await coro(*args, **kwargs)
+            except psycopg2.errors.OperationalError as exc:
+                if "server closed the connection unexpectedly" in str(exc):
+                    final_exc = exc
+                    continue
+                raise exc
+        raise exceptions.ConnectorException(
+            "Could not get a valid connection after {} tries".format(max_tries)
+        ) from final_exc
+
+    return wrapped
+
+
+class AiopgConnector(connector.BaseAsyncConnector):
     def __init__(
         self,
         *,
@@ -88,18 +125,18 @@ class AiopgConnector(connector.BaseConnector):
             instead of standard cursor. There is no identified use case for changing
             this.
         maxsize : ``int``
-            Passed to aiopg. Cannot be lower than 2, otherwise worker won't be
-            functioning normally (one connection for listen/notify, one for executing
-            tasks).
+            Passed to aiopg. If value is 1, then listen/notify feature will be
+            deactivated.
         minsize : ``int``
             Passed to aiopg. Initial connections are not opened when the connector
             is created, but at first use of the pool.
         """
         self._pool: Optional[aiopg.Pool] = None
+        self._pool_externally_set: bool = False
         self.json_dumps = json_dumps
         self.json_loads = json_loads
         self._pool_args = self._adapt_pool_args(kwargs, json_loads)
-        self._lock = asyncio.Lock()
+        self._lock: Optional[asyncio.Lock] = None
 
     @staticmethod
     def _adapt_pool_args(
@@ -125,8 +162,6 @@ class AiopgConnector(connector.BaseConnector):
             "on_connect": on_connect,
             "cursor_factory": RealDictCursor,
         }
-        if "maxsize" in pool_args:
-            pool_args["maxsize"] = max(2, pool_args["maxsize"])
 
         final_args.update(pool_args)
         return final_args
@@ -141,6 +176,17 @@ class AiopgConnector(connector.BaseConnector):
             await self._pool.wait_closed()
             self._pool = None
 
+    def __del__(self):
+        if self._pool and not self._pool_externally_set:
+            # This one deserves a comment. Aiopg will close the free connections upon
+            # __del__ but warns when doing so, so we'll be doing it ourselves. Plus,
+            # there's no official sync method for closing all the connections in a pool.
+            # This is a hack, and if something breaks around connection closing in the
+            # future, it's a good idea to start looking here.
+            self._pool.terminate()
+            while self._pool._free:
+                self._pool._free.popleft().close()
+
     def _wrap_json(self, arguments: Dict[str, Any]):
         return {
             key: Json(value, dumps=self.json_dumps)
@@ -154,20 +200,24 @@ class AiopgConnector(connector.BaseConnector):
     async def _create_pool(pool_args: Dict[str, Any]) -> aiopg.Pool:
         return await aiopg.create_pool(**pool_args)
 
-    def set_pool(self, pool: aiopg.Pool) -> None:
+    def set_pool(self, pool: aiopg.Pool, *, _external: bool = True) -> None:
         """
         Set the connection pool. Raises an exception if the pool is already set.
         """
+
         if self._pool:
             raise exceptions.PoolAlreadySet
+        self._pool_externally_set = _external
         self._pool = pool
 
     async def _get_pool(self) -> aiopg.Pool:
         if self._pool:
             return self._pool
+        if not self._lock:
+            self._lock = asyncio.Lock()
         async with self._lock:
             if not self._pool:
-                self.set_pool(await self._create_pool(self._pool_args))
+                self.set_pool(await self._create_pool(self._pool_args), _external=False)
         return self._pool
 
     # Pools and single connections do not exactly share their cursor API:
@@ -177,12 +227,14 @@ class AiopgConnector(connector.BaseConnector):
     # a pool or from a connection
 
     @wrap_exceptions
-    async def execute_query(self, query: str, **arguments: Any) -> None:
+    @wrap_query_exceptions
+    async def execute_query_async(self, query: str, **arguments: Any) -> None:
         pool = await self._get_pool()
         with await pool.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
 
     @wrap_exceptions
+    @wrap_query_exceptions
     async def _execute_query_connection(
         self, query: str, connection: aiopg.Connection, **arguments: Any,
     ) -> None:
@@ -190,7 +242,10 @@ class AiopgConnector(connector.BaseConnector):
             await cursor.execute(query, self._wrap_json(arguments))
 
     @wrap_exceptions
-    async def execute_query_one(self, query: str, **arguments: Any) -> Dict[str, Any]:
+    @wrap_query_exceptions
+    async def execute_query_one_async(
+        self, query: str, **arguments: Any
+    ) -> Dict[str, Any]:
         pool = await self._get_pool()
         with await pool.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
@@ -198,7 +253,8 @@ class AiopgConnector(connector.BaseConnector):
             return await cursor.fetchone()
 
     @wrap_exceptions
-    async def execute_query_all(
+    @wrap_query_exceptions
+    async def execute_query_all_async(
         self, query: str, **arguments: Any
     ) -> List[Dict[str, Any]]:
         pool = await self._get_pool()
@@ -218,10 +274,18 @@ class AiopgConnector(connector.BaseConnector):
     @wrap_exceptions
     async def listen_notify(
         self, event: asyncio.Event, channels: Iterable[str]
-    ) -> NoReturn:
+    ) -> None:
         pool = await self._get_pool()
         # We need to acquire a dedicated connection, and use the listen
         # query
+        if pool.maxsize == 1:
+            logger.warning(
+                "Listen/Notify capabilities disabled because maximum pool size"
+                "is set to 1",
+                extra={"action": "listen_notify_disabled"},
+            )
+            return
+
         while True:
             async with pool.acquire() as connection:
                 for channel_name in channels:
