@@ -1,6 +1,6 @@
 import functools
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Optional, Set
+from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional, Set
 
 from procrastinate import admin
 from procrastinate import connector as connector_module
@@ -56,13 +56,13 @@ class App:
         connector: connector_module.BaseConnector,
         import_paths: Optional[Iterable[str]] = None,
         worker_defaults: Optional[Dict] = None,
+        periodic_defaults: Optional[Dict] = None,
     ):
         """
         Parameters
         ----------
         connector :
-            Instance of a subclass of :py:class:`procrastinate.connector.BaseConnector`,
-            typically `AiopgConnector`. It will be responsible for all communications
+            Typically an `AiopgConnector`. It will be responsible for all communications
             with the database. Mandatory.
         import_paths :
             List of python dotted paths of modules to import, to make sure
@@ -78,15 +78,29 @@ class App:
         worker_defaults :
             All the values passed here will override the default values sent when
             launching a worker. See `App.run_worker` for details.
+        periodic_defaults :
+            Parameters for fine tuning the periodic tasks deferrer. Available
+            parameters are:
+
+            - ``max_delay``: ``float``, in seconds, controls how long after the planned
+              launch of a periodic task a deferrer can launch the task. Thanks to this
+              parameter, when deploying a new periodic task, it's usually not deferred
+              until its next scheduled time. Defaults to 10 minutes.
         """
+        from procrastinate import periodic
+
         self.connector = connector
         self.tasks: Dict[str, "tasks.Task"] = {}
         self.builtin_tasks: Dict[str, "tasks.Task"] = {}
         self.queues: Set[str] = set()
         self.import_paths = import_paths or []
         self.worker_defaults = worker_defaults or {}
+        periodic_defaults = periodic_defaults or {}
 
         self.job_store = store.JobStore(connector=self.connector)
+        self.periodic_deferrer = periodic.PeriodicDeferrer(
+            job_store=self.job_store, **periodic_defaults
+        )
 
         self._register_builtin_tasks()
 
@@ -96,6 +110,7 @@ class App:
         *,
         queue: str = jobs.DEFAULT_QUEUE,
         name: Optional[str] = None,
+        aliases: Optional[List[str]] = None,
         retry: retry_module.RetryValue = False,
         pass_context: bool = False,
     ) -> Any:
@@ -129,6 +144,11 @@ class App:
             if the function is nested or dynamically defined, it is important to give
             it a unique name, and to make sure the module that defines this function
             is listed in the ``import_paths`` of the `procrastinate.App`.
+        aliases:
+            Additional names for the task.
+            The main use case is to gracefully rename tasks by moving the old
+            name to aliases (these tasks can have been scheduled in a distant
+            future, so the aliases can remain for a long time).
         retry :
             Details how to auto-retry the task if it fails. Can be:
 
@@ -151,6 +171,7 @@ class App:
                 app=self,
                 queue=queue,
                 name=name,
+                aliases=aliases,
                 retry=retry,
                 pass_context=pass_context,
             )
@@ -163,8 +184,22 @@ class App:
 
         return _wrap(_func)  # Called as @app.task
 
+    def periodic(self, *, cron: str):
+        """
+        Task decorator, marks task as being scheduled for periodic deferring (see
+        `howto/cron`).
+
+        Parameters
+        ----------
+        cron :
+            Cron-like string. Optionally add a 6th column for seconds.
+        """
+        return self.periodic_deferrer.periodic_decorator(cron=cron)
+
     def _register(self, task: "tasks.Task") -> None:
         self.tasks[task.name] = task
+        for alias in task.aliases:
+            self.tasks[alias] = task
         queue = task.queue
         if queue not in self.queues:
             logger.debug(
@@ -224,7 +259,7 @@ class App:
         Run a worker. This worker will run in the foreground and execute the jobs in the
         provided queues. If wait is True, the function will not
         return until the worker stops (most probably when it receives a stop signal).
-        The default values of all parameters presented here can be overriden at the
+        The default values of all parameters presented here can be overridden at the
         `App` level.
 
         Parameters
@@ -235,7 +270,7 @@ class App:
         wait : ``bool``
             If False, the worker will terminate as soon as it has caught up with the
             queues. If True, the worker will work until it is stopped by a signal
-            (ctrl+c, SIGINT, SIGTERM) (defaults to ``True``).
+            (``ctrl+c``, ``SIGINT``, ``SIGTERM``) (defaults to ``True``).
         concurrency : ``int``
             Indicates how many asynchronous jobs the worker can run in parallel.
             Do not use concurrency if you have synchronous blocking tasks.
@@ -250,6 +285,11 @@ class App:
             Raising this parameter can lower the rate of workers making queries to the
             database for requesting jobs.
             (defaults to 5.0)
+        listen_notify : ``bool``
+            If ``True``, worker will dedicate a connection from the pool to listening to
+            database events, notifying of newly available jobs. If ``False``, workers
+            will just poll the database periodically (see ``timeout``). (defaults to
+            True)
         """
         self.perform_import_paths()
         worker = self._worker(**kwargs)
