@@ -3,12 +3,10 @@ import pathlib
 from importlib_resources import read_text
 
 from procrastinate import connector as connector_module
-from procrastinate import utils
 
 migrations_path = pathlib.Path(__file__).parent / "sql" / "migrations"
 
 
-@utils.add_sync_api
 class SchemaManager:
     def __init__(self, connector: connector_module.BaseConnector):
 
@@ -22,6 +20,6 @@ class SchemaManager:
     def get_migrations_path() -> str:
         return str(migrations_path)
 
-    async def apply_schema_async(self) -> None:
+    def apply_schema(self) -> None:
         queries = self.get_schema()
-        await self.connector.execute_query(query=queries)
+        self.connector.execute_query(query=queries)
