@@ -2,6 +2,7 @@
 
 CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;
 
+-- Enums
 
 CREATE TYPE procrastinate_job_status AS ENUM (
     'todo',  -- The job is queued
@@ -20,6 +21,8 @@ CREATE TYPE procrastinate_job_event_type AS ENUM (
     'scheduled' -- not an event transition, but recording when a task is scheduled for
 );
 
+-- Tables
+
 CREATE TABLE procrastinate_jobs (
     id bigserial PRIMARY KEY,
     queue_name character varying(128) NOT NULL,
@@ -32,8 +35,13 @@ CREATE TABLE procrastinate_jobs (
     attempts integer DEFAULT 0 NOT NULL
 );
 
--- this prevents from having several jobs with the same queueing lock in the "todo" state
-CREATE UNIQUE INDEX procrastinate_jobs_queueing_lock_idx ON procrastinate_jobs (queueing_lock) WHERE status = 'todo';
+CREATE TABLE procrastinate_periodic_defers (
+    id bigserial PRIMARY KEY,
+    task_name character varying(128) NOT NULL,
+    defer_timestamp bigint,
+    job_id bigint REFERENCES procrastinate_jobs(id) NULL,
+    UNIQUE (task_name, defer_timestamp)
+);
 
 CREATE TABLE procrastinate_events (
     id BIGSERIAL PRIMARY KEY,
@@ -42,9 +50,83 @@ CREATE TABLE procrastinate_events (
     at timestamp with time zone DEFAULT NOW() NULL
 );
 
-CREATE TABLE procrastinate_job_locks (
-    object text PRIMARY KEY
-);
+-- Contraints & Indices
+
+-- this prevents from having several jobs with the same queueing lock in the "todo" state
+CREATE UNIQUE INDEX procrastinate_jobs_queueing_lock_idx ON procrastinate_jobs (queueing_lock) WHERE status = 'todo';
+-- this prevents from having several jobs with the same lock in the "doing" state
+CREATE UNIQUE INDEX procrastinate_jobs_lock_idx ON procrastinate_jobs (lock) WHERE status = 'doing';
+
+CREATE INDEX procrastinate_jobs_queue_name_idx ON procrastinate_jobs(queue_name);
+CREATE INDEX procrastinate_events_job_id_fkey ON procrastinate_events(job_id);
+
+-- Functions
+
+CREATE FUNCTION procrastinate_defer_job(
+    queue_name character varying,
+    task_name character varying,
+    lock text,
+    queueing_lock text,
+    args jsonb,
+    scheduled_at timestamp with time zone
+) RETURNS bigint
+    LANGUAGE plpgsql
+    AS $$
+DECLARE
+	job_id bigint;
+BEGIN
+    INSERT INTO procrastinate_jobs (queue_name, task_name, lock, queueing_lock, args, scheduled_at)
+    VALUES (queue_name, task_name, lock, queueing_lock, args, scheduled_at)
+    RETURNING id INTO job_id;
+
+    RETURN job_id;
+END;
+$$;
+
+CREATE FUNCTION procrastinate_defer_periodic_job(
+    _queue_name character varying,
+    _task_name character varying,
+    _defer_timestamp bigint
+) RETURNS bigint
+    LANGUAGE plpgsql
+    AS $$
+DECLARE
+	_job_id bigint;
+	_defer_id bigint;
+BEGIN
+
+    INSERT
+        INTO procrastinate_periodic_defers (task_name, defer_timestamp)
+        VALUES (_task_name, _defer_timestamp)
+        ON CONFLICT DO NOTHING
+        RETURNING id into _defer_id;
+
+    IF _defer_id IS NULL THEN
+        RETURN NULL;
+    END IF;
+
+    UPDATE procrastinate_periodic_defers
+        SET job_id = procrastinate_defer_job(
+                _queue_name,
+                _task_name,
+                NULL,
+                NULL,
+                ('{"timestamp": ' || _defer_timestamp || '}')::jsonb,
+                NULL
+            )
+        WHERE id = _defer_id
+        RETURNING job_id INTO _job_id;
+
+    DELETE
+        FROM procrastinate_periodic_defers
+        WHERE
+            _job_id IS NOT NULL
+            AND procrastinate_periodic_defers.task_name = _task_name
+            AND procrastinate_periodic_defers.defer_timestamp < _defer_timestamp;
+
+    RETURN _job_id;
+END;
+$$;
 
 CREATE FUNCTION procrastinate_fetch_job(target_queue_names character varying[]) RETURNS procrastinate_jobs
     LANGUAGE plpgsql
@@ -52,28 +134,28 @@ CREATE FUNCTION procrastinate_fetch_job(target_queue_names character varying[])
 DECLARE
 	found_jobs procrastinate_jobs;
 BEGIN
-	WITH potential_job AS (
-		SELECT procrastinate_jobs.*
-			FROM procrastinate_jobs
-			LEFT JOIN procrastinate_job_locks ON procrastinate_job_locks.object = procrastinate_jobs.lock
-			WHERE (target_queue_names IS NULL OR queue_name = ANY( target_queue_names ))
-			  AND procrastinate_job_locks.object IS NULL
-			  AND status = 'todo'
-			  AND (scheduled_at IS NULL OR scheduled_at <= now())
-            ORDER BY id ASC
-			FOR UPDATE OF procrastinate_jobs SKIP LOCKED LIMIT 1
-	), lock_object AS (
-		INSERT INTO procrastinate_job_locks
-			SELECT lock FROM potential_job
-            ON CONFLICT DO NOTHING
-            RETURNING object
-	)
-	UPDATE procrastinate_jobs
-		SET status = 'doing'
-		FROM potential_job, lock_object
-        WHERE lock_object.object IS NOT NULL
-		AND procrastinate_jobs.id = potential_job.id
-		RETURNING procrastinate_jobs.* INTO found_jobs;
+    WITH candidate AS (
+        SELECT jobs.*
+            FROM procrastinate_jobs AS jobs
+            WHERE
+                -- reject the job if its lock has earlier jobs
+                NOT EXISTS (
+                    SELECT 1
+                        FROM procrastinate_jobs AS earlier_jobs
+                        WHERE earlier_jobs.lock = jobs.lock
+                            AND earlier_jobs.status IN ('todo', 'doing')
+                            AND earlier_jobs.id < jobs.id)
+                AND jobs.status = 'todo'
+                AND (target_queue_names IS NULL OR jobs.queue_name = ANY( target_queue_names ))
+                AND (jobs.scheduled_at IS NULL OR jobs.scheduled_at <= now())
+            ORDER BY jobs.id ASC LIMIT 1
+            FOR UPDATE OF jobs SKIP LOCKED
+    )
+    UPDATE procrastinate_jobs
+        SET status = 'doing'
+        FROM candidate
+        WHERE procrastinate_jobs.id = candidate.id
+        RETURNING procrastinate_jobs.* INTO found_jobs;
 
 	RETURN found_jobs;
 END;
@@ -83,14 +165,11 @@ CREATE FUNCTION procrastinate_finish_job(job_id integer, end_status procrastinat
     LANGUAGE plpgsql
     AS $$
 BEGIN
-	WITH finished_job AS (
-		UPDATE procrastinate_jobs
-        SET status = end_status,
-            attempts = attempts + 1,
-            scheduled_at = COALESCE(next_scheduled_at, scheduled_at)
-        WHERE id = job_id RETURNING lock
-	)
-	DELETE FROM procrastinate_job_locks WHERE object = (SELECT lock FROM finished_job);
+    UPDATE procrastinate_jobs
+    SET status = end_status,
+        attempts = attempts + 1,
+        scheduled_at = COALESCE(next_scheduled_at, scheduled_at)
+    WHERE id = job_id;
 END;
 $$;
 
@@ -160,7 +239,7 @@ BEGIN
 END;
 $$;
 
-CREATE INDEX ON procrastinate_jobs(queue_name);
+-- Triggers
 
 CREATE TRIGGER procrastinate_jobs_notify_queue
     AFTER INSERT ON procrastinate_jobs
@@ -181,5 +260,3 @@ CREATE TRIGGER procrastinate_trigger_scheduled_events
     AFTER UPDATE OR INSERT ON procrastinate_jobs
     FOR EACH ROW WHEN ((new.scheduled_at IS NOT NULL AND new.status = 'todo'::procrastinate_job_status))
     EXECUTE PROCEDURE procrastinate_trigger_scheduled_events_procedure();
-
-CREATE INDEX procrastinate_events_job_id_fkey ON procrastinate_events(job_id);
