@@ -1,7 +1,7 @@
 import asyncio
 import datetime
 import uuid
-from typing import Iterable, Optional
+from typing import Any, Dict, Iterable, Optional
 
 from procrastinate import connector, exceptions, jobs, sql
 
@@ -14,33 +14,64 @@ def get_channel_for_queues(queues: Optional[Iterable[str]] = None) -> Iterable[s
 
 
 class JobStore:
-    def __init__(self, connector: connector.BaseConnector):
+    def __init__(
+        self, connector: connector.BaseConnector,
+    ):
         self.connector = connector
 
-    async def defer_job(self, job: jobs.Job) -> int:
+    async def defer_job_async(self, job: jobs.Job) -> int:
+        # Make sure this code stays synchronized with .defer_job()
         try:
-            result = await self.connector.execute_query_one(
-                query=sql.queries["defer_job"],
-                task_name=job.task_name,
-                lock=job.lock or str(uuid.uuid4()),
-                queueing_lock=job.queueing_lock,
-                args=job.task_kwargs,
-                scheduled_at=job.scheduled_at,
-                queue=job.queue,
+            result = await self.connector.execute_query_one_async(
+                **self._defer_job_query_kwargs(job=job)
             )
         except exceptions.UniqueViolation as exc:
-            if exc.constraint_name == connector.QUEUEING_LOCK_CONSTRAINT:
-                raise exceptions.AlreadyEnqueued(
-                    "Job cannot be enqueued: there is already a job in the queue "
-                    f"with the lock {job.queueing_lock}"
-                ) from exc
-            raise
+            self._raise_already_enqueued(exc=exc, job=job)
 
         return result["id"]
 
+    def defer_job(self, job: jobs.Job) -> int:
+        try:
+            result = self.connector.execute_query_one(
+                **self._defer_job_query_kwargs(job=job)
+            )
+        except exceptions.UniqueViolation as exc:
+            self._raise_already_enqueued(exc=exc, job=job)
+
+        return result["id"]
+
+    def _defer_job_query_kwargs(self, job: jobs.Job) -> Dict[str, Any]:
+
+        return {
+            "query": sql.queries["defer_job"],
+            "task_name": job.task_name,
+            "lock": job.lock or str(uuid.uuid4()),
+            "queueing_lock": job.queueing_lock,
+            "args": job.task_kwargs,
+            "scheduled_at": job.scheduled_at,
+            "queue": job.queue,
+        }
+
+    def _raise_already_enqueued(self, exc: exceptions.UniqueViolation, job: jobs.Job):
+        if exc.constraint_name == connector.QUEUEING_LOCK_CONSTRAINT:
+            raise exceptions.AlreadyEnqueued(
+                "Job cannot be enqueued: there is already a job in the queue "
+                f"with the lock {job.queueing_lock}"
+            ) from exc
+        raise exc
+
+    async def defer_periodic_job(self, task, defer_timestamp) -> Optional[int]:
+        result = await self.connector.execute_query_one_async(
+            query=sql.queries["defer_periodic_job"],
+            queue=task.queue,
+            task_name=task.name,
+            defer_timestamp=defer_timestamp,
+        )
+        return result["id"]
+
     async def fetch_job(self, queues: Optional[Iterable[str]]) -> Optional[jobs.Job]:
 
-        row = await self.connector.execute_query_one(
+        row = await self.connector.execute_query_one_async(
             query=sql.queries["fetch_job"], queues=queues
         )
 
@@ -58,7 +89,7 @@ class JobStore:
         task_name: Optional[str] = None,
     ) -> Iterable[jobs.Job]:
 
-        rows = await self.connector.execute_query_all(
+        rows = await self.connector.execute_query_all_async(
             query=sql.queries["select_stalled_jobs"],
             nb_seconds=nb_seconds,
             queue=queue,
@@ -78,7 +109,7 @@ class JobStore:
         else:
             statuses = [jobs.Status.SUCCEEDED.value, jobs.Status.FAILED.value]
 
-        await self.connector.execute_query(
+        await self.connector.execute_query_async(
             query=sql.queries["delete_old_jobs"],
             nb_hours=nb_hours,
             queue=queue,
@@ -92,7 +123,7 @@ class JobStore:
         scheduled_at: Optional[datetime.datetime] = None,
     ) -> None:
         assert job.id  # TODO remove this
-        await self.connector.execute_query(
+        await self.connector.execute_query_async(
             query=sql.queries["finish_job"],
             job_id=job.id,
             status=status.value,
