@@ -1,6 +1,6 @@
 import datetime
 import logging
-from typing import Any, Callable, Dict, Optional
+from typing import Any, Callable, Dict, List, Optional
 
 import pendulum
 
@@ -52,7 +52,6 @@ def configure_task(
     )
 
 
-@utils.add_sync_api
 class Task:
     """
     A task is a function that should be executed later. It is linked to a
@@ -64,6 +63,8 @@ class Task:
         Default queue to send deferred jobs to.
     name : ``str``
         Name of the task, usually the dotted path of the decorated function.
+    aliases : ``List[str]``
+        Additional names for the task.
     retry_strategy : `RetryStrategy`
         Value indicating the retry conditions in case of
         :py:class:`procrastinate.jobs.Job` error.
@@ -79,12 +80,14 @@ class Task:
         app: app.App,
         queue: str,
         name: Optional[str] = None,
+        aliases: Optional[List[str]] = None,
         retry: retry_module.RetryValue = False,
         pass_context: bool = False,
     ):
         self.queue = queue
         self.app = app
         self.func: Callable = func
+        self.aliases = aliases if aliases else []
         self.retry_strategy = retry_module.get_retry_strategy(retry)
         self.name: str = name if name else self.full_path
         self.pass_context = pass_context
@@ -94,7 +97,7 @@ class Task:
 
     @property
     def full_path(self) -> str:
-        return f"{self.func.__module__}.{self.func.__name__}"
+        return utils.get_full_path(self.func)
 
     async def defer_async(self, **task_kwargs: types.JSONValue) -> int:
         """
@@ -102,9 +105,15 @@ class Task:
         The job will be created with default parameters, if you want to better
         specify when and how to launch this job, see `Task.configure`.
         """
-        job_id = await self.configure().defer_async(**task_kwargs)
+        return await self.configure().defer_async(**task_kwargs)
 
-        return job_id
+    def defer(self, **task_kwargs: types.JSONValue) -> int:
+        """
+        Create a job from this task and the given arguments.
+        The job will be created with default parameters, if you want to better
+        specify when and how to launch this job, see `Task.configure`.
+        """
+        return self.configure().defer(**task_kwargs)
 
     def configure(
         self,
