@@ -12,7 +12,7 @@ JobRow = Dict[str, Any]
 EventRow = Dict[str, Any]
 
 
-class InMemoryConnector(connector.BaseConnector):
+class InMemoryConnector(connector.BaseAsyncConnector):
     """
     An InMemoryConnector may be used for testing only. Tasks are not
     persisted and will be lost when the process ends.
@@ -42,6 +42,7 @@ class InMemoryConnector(connector.BaseConnector):
         self.queries: List[Tuple[str, Dict[str, Any]]] = []
         self.notify_event = None
         self.notify_channels = []
+        self.periodic_defers: Dict[str, int] = {}
 
     def generic_execute(self, query, suffix, **arguments) -> Any:
         """
@@ -56,13 +57,15 @@ class InMemoryConnector(connector.BaseConnector):
     def make_dynamic_query(self, query, **identifiers: str) -> str:
         return query.format(**identifiers)
 
-    async def execute_query(self, query: str, **arguments: Any) -> None:
+    async def execute_query_async(self, query: str, **arguments: Any) -> None:
         self.generic_execute(query, "run", **arguments)
 
-    async def execute_query_one(self, query: str, **arguments: Any) -> Dict[str, Any]:
+    async def execute_query_one_async(
+        self, query: str, **arguments: Any
+    ) -> Dict[str, Any]:
         return self.generic_execute(query, "one", **arguments)
 
-    async def execute_query_all(
+    async def execute_query_all_async(
         self, query: str, **arguments: Any
     ) -> List[Dict[str, Any]]:
         return self.generic_execute(query, "all", **arguments)
@@ -73,6 +76,12 @@ class InMemoryConnector(connector.BaseConnector):
         self.notify_event = event
         self.notify_channels = channels
 
+    def close(self) -> None:
+        pass
+
+    async def close_async(self) -> None:
+        pass
+
     # End of BaseConnector methods
 
     def defer_job_one(
@@ -109,6 +118,19 @@ class InMemoryConnector(connector.BaseConnector):
                 self.notify_event.set()
         return job_row
 
+    def defer_periodic_job_one(self, queue, task_name, defer_timestamp):
+        if self.periodic_defers.get(task_name) == defer_timestamp:
+            return {"id": None}
+        self.periodic_defers[task_name] = defer_timestamp
+        return self.defer_job_one(
+            task_name=task_name,
+            queue=queue,
+            lock=None,
+            queueing_lock=None,
+            args={"timestamp": defer_timestamp},
+            scheduled_at=None,
+        )
+
     @property
     def current_locks(self) -> Iterable[str]:
         return {job["lock"] for job in self.jobs.values() if job["status"] == "doing"}
