@@ -1,7 +1,10 @@
 import asyncio
+import contextlib
 import functools
 import importlib
+import inspect
 import logging
+import pathlib
 import types
 from typing import Any, Awaitable, Iterable, Optional, Type, TypeVar
 
@@ -82,6 +85,66 @@ def wrap_one(cls: Type, attribute_name: str):
     if attribute_name.startswith("_") or not attribute_name.endswith(suffix):
         return
 
+    attribute, function = get_raw_method(cls=cls, attribute_name=attribute_name)
+
+    # Keep only async def methods
+    if not asyncio.iscoroutinefunction(function):
+        return
+
+    if isinstance(attribute, types.FunctionType):  # classic method
+        method_type = "method"
+    elif isinstance(attribute, classmethod):
+        method_type = "classmethod"
+    elif isinstance(attribute, staticmethod):
+        method_type = "staticmethod"
+    else:
+        raise ValueError(f"Invalid object of type {type(attribute)}")
+
+    attribute.__doc__ = attribute.__doc__ or ""
+
+    # Create a wrapper that will call the method in a run_until_complete
+    @functools.wraps(function)
+    def wrapper(*args, **kwargs):
+        if method_type == "method":
+            final_class = type(args[0])
+        elif method_type == "classmethod":
+            final_class = args[0]
+        else:
+            final_class = cls
+
+        _, function = get_raw_method(cls=final_class, attribute_name=attribute_name)
+
+        awaitable = function(*args, **kwargs)
+        return sync_await(awaitable=awaitable)
+
+    sync_name = attribute_name[: -len(suffix)]
+    attribute.__doc__ += ASYNC_ADDENDUM.format(sync_name)
+
+    final_wrapper: Any
+    if method_type == "method":
+        final_wrapper = wrapper
+    elif method_type == "classmethod":
+        final_wrapper = classmethod(wrapper)
+    else:
+        final_wrapper = staticmethod(wrapper)
+
+    # Save this new method on the class
+    wrapper.__name__ = sync_name
+    final_wrapper.__doc__ += SYNC_ADDENDUM.format(attribute_name)
+    setattr(cls, sync_name, final_wrapper)
+
+
+def get_raw_method(cls: Type, attribute_name: str):
+    """
+    Extract a method from the class, without triggering the descriptor.
+    Return 2 objects:
+
+    - the method itself stored on the class (which may be a function, a classmethod or
+      a staticmethod)
+    - The real function underneath (the same function as above for a normal method,
+      and the wrapped function for static and class methods).
+
+    """
     # Methods are descriptors so using getattr here will not give us the real method
     cls_vars = vars(cls)
     attribute = cls_vars[attribute_name]
@@ -90,35 +153,7 @@ def wrap_one(cls: Type, attribute_name: str):
     # async, is stored in __func__.
     wrapped = getattr(attribute, "__func__", attribute)
 
-    # Keep only async def methods
-    if not asyncio.iscoroutinefunction(wrapped):
-        return
-
-    attribute.__doc__ = attribute.__doc__ or ""
-
-    # Create a wrapper that will call the method in a run_until_complete
-    @functools.wraps(wrapped)
-    def wrapper(*args, **kwargs):
-        awaitable = wrapped(*args, **kwargs)
-        return sync_await(awaitable=awaitable)
-
-    sync_name = attribute_name[: -len(suffix)]
-    attribute.__doc__ += ASYNC_ADDENDUM.format(sync_name)
-
-    final_wrapper: Any
-    if isinstance(attribute, types.FunctionType):  # classic method
-        final_wrapper = wrapper
-    elif isinstance(attribute, classmethod):
-        final_wrapper = classmethod(wrapper)
-    elif isinstance(attribute, staticmethod):
-        final_wrapper = staticmethod(wrapper)
-    else:
-        raise ValueError(f"Invalid object of type {type(attribute)}")
-
-    # Save this new method on the class
-    name = wrapper.__name__ = sync_name
-    final_wrapper.__doc__ += SYNC_ADDENDUM.format(attribute_name)
-    setattr(cls, name, final_wrapper)
+    return attribute, wrapped
 
 
 def sync_await(awaitable: Awaitable[T]) -> T:
@@ -141,3 +176,60 @@ def causes(exc: Optional[BaseException]):
     while exc:
         yield exc
         exc = exc.__cause__ or exc.__context__
+
+
+def _get_module_name(obj: Any) -> str:
+    module_name = obj.__module__
+    if module_name != "__main__":
+        return module_name
+
+    module = inspect.getmodule(obj)
+    # obj could be None, or has no __file__ if in an interactive shell
+    # in which case, there's not a lot we can do.
+    if not module or not hasattr(module, "__file__"):
+        return module_name
+
+    path = pathlib.Path(module.__file__)
+    # If the path is absolute, it probably means __main__ is an executable from an
+    # installed package
+    if path.is_absolute():
+        return module_name
+
+    # Creating the dotted path from the path
+    return ".".join([*path.parts[:-1], path.stem])
+
+
+def get_full_path(obj: Any) -> str:
+
+    return f"{_get_module_name(obj)}.{obj.__name__}"
+
+
+@contextlib.contextmanager
+def task_context(awaitable: Awaitable, name: str):
+    """
+    Take an awaitable, return a context manager.
+
+    On enter, launch the awaitable as a task that will execute in parallel in the
+    event loop. On exit, cancel the task (and log). If the task ends with an exception
+    log it.
+
+    A name is required for logging purposes.
+    """
+    nice_name = name.replace("_", " ").title()
+
+    async def wrapper():
+        try:
+            logger.debug(f"Started {nice_name}", extra={"action": f"{name}_start"})
+            await awaitable
+        except asyncio.CancelledError:
+            logger.debug(f"Stopped {nice_name}", extra={"action": f"{name}_stop"})
+            raise
+
+        except Exception:
+            logger.exception(f"{nice_name} error", extra={"action": f"{name}_error"})
+
+    try:
+        task = asyncio.ensure_future(wrapper())
+        yield task
+    finally:
+        task.cancel()
