@@ -4,7 +4,7 @@ import logging
 import time
 from typing import Dict, Iterable, Optional, Set, Union
 
-from procrastinate import app, exceptions, job_context, jobs, signals, tasks
+from procrastinate import app, exceptions, job_context, jobs, signals, tasks, utils
 
 logger = logging.getLogger(__name__)
 
@@ -23,6 +23,7 @@ class Worker:
         concurrency: int = 1,
         wait: bool = True,
         timeout: float = WORKER_TIMEOUT,
+        listen_notify: bool = True,
     ):
         self.app = app
         self.queues = queues
@@ -31,6 +32,7 @@ class Worker:
 
         self.timeout = timeout
         self.wait = wait
+        self.listen_notify = listen_notify
 
         # Handling the info about the currently running task.
         self.known_missing_tasks: Set[str] = set()
@@ -69,16 +71,19 @@ class Worker:
 
         return context
 
-    @contextlib.contextmanager
     def listener(self):
         assert self.notify_event
-        notifier = asyncio.ensure_future(
-            self.job_store.listen_for_jobs(event=self.notify_event, queues=self.queues)
+        return utils.task_context(
+            awaitable=self.job_store.listen_for_jobs(
+                event=self.notify_event, queues=self.queues
+            ),
+            name="listener",
+        )
+
+    def periodic_deferrer(self):
+        return utils.task_context(
+            awaitable=self.app.periodic_deferrer.worker(), name="periodic_deferrer",
         )
-        try:
-            yield
-        finally:
-            notifier.cancel()
 
     async def run(self) -> None:
         self.notify_event = asyncio.Event()
@@ -91,7 +96,13 @@ class Worker:
             ),
         )
 
-        with self.listener(), signals.on_stop(self.stop):
+        with contextlib.ExitStack() as stack:
+            if self.wait and self.listen_notify:
+                stack.enter_context(self.listener())
+
+            stack.enter_context(self.periodic_deferrer())
+            stack.enter_context(signals.on_stop(self.stop))
+
             await asyncio.gather(
                 *(
                     self.single_worker(worker_id=worker_id)
@@ -225,20 +236,20 @@ class Worker:
 
         except Exception as e:
             task_result = None
-            log_title = "Job error"
+            log_title = "Error"
             log_action = "job_error"
             log_level = logging.ERROR
             exc_info = e
 
             retry_exception = task.get_retry_exception(exception=e, job=job)
             if retry_exception:
-                log_title = "Job error, to retry"
+                log_title = "Error, to retry"
                 log_action = "job_error_retry"
                 raise retry_exception from e
             raise exceptions.JobError() from e
 
         else:
-            log_title = "Job success"
+            log_title = "Success"
             log_action = "job_success"
             log_level = logging.INFO
             exc_info = False
@@ -255,7 +266,12 @@ class Worker:
 
             extra = context.log_extra(action=log_action)
 
-            text = f"{log_title} - Job {job.call_string} " f"in {duration:.3f} s"
+            text = (
+                f"Job {job.call_string} ended with status: {log_title}, "
+                f"lasted {duration:.3f} s"
+            )
+            if task_result:
+                text += f" - Result: {task_result}"[:250]
             self.logger.log(log_level, text, extra=extra, exc_info=exc_info)
 
     def stop(self):
